==> jruby1.1_1.1.5.orig.tar.gz: jruby-1.1.5/src/org/jruby/ast/HashNode.java <==
/*
 ***** BEGIN LICENSE BLOCK *****
 * Version: CPL 1.0/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Common Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.eclipse.org/legal/cpl-v10.html
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Copyright (C) 2001-2002 Jan Arne Petersen <jpetersen@uni-bonn.de>
 * Copyright (C) 2001-2002 Benoit Cerrina <b.cerrina@wanadoo.fr>
 * Copyright (C) 2002 Anders Bengtsson <ndrsbngtssn@yahoo.se>
 * Copyright (C) 2004 Thomas E Enebo <enebo@acm.org>
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the CPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the CPL, the GPL or the LGPL.
 ***** END LICENSE BLOCK *****/
package org.jruby.ast;

import java.util.List;

import org.jruby.Ruby;
import org.jruby.RubyHash;
import org.jruby.ast.visitor.NodeVisitor;
import org.jruby.evaluator.Instruction;
import org.jruby.lexer.yacc.ISourcePosition;
import org.jruby.runtime.Block;
import org.jruby.runtime.ThreadContext;
import org.jruby.runtime.builtin.IRubyObject;

/**
 * A Literal Hash that can represent either a {a=&amp;b, c=&amp;d} type expression or the list 
 * of default values in a method call.
 */
public class HashNode extends Node {
    private final ListNode listNode;
    
    public HashNode(ISourcePosition position, ListNode listNode) {
        super(position, NodeType.HASHNODE);
        this.listNode = listNode;
    }

    /**
     * Accept for the visitor pattern.
     * @param iVisitor the visitor
     **/
    public Instruction accept(NodeVisitor iVisitor) {
        return iVisitor.visitHashNode(this);
    }

    /**
     * Gets the listNode.
     * @return Returns a IListNode
     */
    public ListNode getListNode() {
        return listNode;
    }
    
    public List<Node> childNodes() {
        return createList(listNode);
    }
    
    @Override
    public IRubyObject interpret(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
        RubyHash hash = RubyHash.newHash(runtime);
        
        ListNode listNode = this.listNode;
        if (listNode != null) {
            int size = listNode.size();
   
            for (int i = 0; i < size;) {
                // insert all nodes in sequence, hash them in the final instruction
                // KEY
                IRubyObject key = listNode.get(i++).interpret(runtime, context, self, aBlock);
                IRubyObject value = listNode.get(i++).interpret(runtime, context, self, aBlock);
   
                hash.fastASet(key, value);
            }
        }
      
        return hash;
    }
}

==> jruby_1.5.1.orig.tar.gz: jruby-1.5.1/src/org/jruby/ast/HashNode.java <==
/*
 ***** BEGIN LICENSE BLOCK *****
 * Version: CPL 1.0/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Common Public
 * License Version 1.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.eclipse.org/legal/cpl-v10.html
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Copyright (C) 2001-2002 Jan Arne Petersen <jpetersen@uni-bonn.de>
 * Copyright (C) 2001-2002 Benoit Cerrina <b.cerrina@wanadoo.fr>
 * Copyright (C) 2002 Anders Bengtsson <ndrsbngtssn@yahoo.se>
 * Copyright (C) 2004 Thomas E Enebo <enebo@acm.org>
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the CPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the CPL, the GPL or the LGPL.
 ***** END LICENSE BLOCK *****/
package org.jruby.ast;

import java.util.List;

import org.jruby.Ruby;
import org.jruby.RubyHash;
import org.jruby.ast.visitor.NodeVisitor;
import org.jruby.lexer.yacc.ISourcePosition;
import org.jruby.runtime.Block;
import org.jruby.runtime.ThreadContext;
import org.jruby.runtime.builtin.IRubyObject;

/**
 * A Literal Hash that can represent either a {a=&amp;b, c=&amp;d} type expression or the list 
 * of default values in a method call.
 */
public class HashNode extends Node {
    private final ListNode listNode;
    
    public HashNode(ISourcePosition position, ListNode listNode) {
        super(position);
        this.listNode = listNode;
    }

    public NodeType getNodeType() {
        return NodeType.HASHNODE;
    }

    /**
     * Accept for the visitor pattern.
     * @param iVisitor the visitor
     **/
    public Object accept(NodeVisitor iVisitor) {
        return iVisitor.visitHashNode(this);
    }

    /**
     * Gets the listNode.
     * @return Returns a IListNode
     */
    public ListNode getListNode() {
        return listNode;
    }
    
    public List<Node> childNodes() {
        return createList(listNode);
    }
    
    @Override
    public IRubyObject interpret(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
        RubyHash hash = RubyHash.newHash(runtime);
        
        ListNode list = this.listNode;
        if (list != null) {
            int size = list.size();
   
            for (int i = 0; i < size;) {
                // insert all nodes in sequence, hash them in the final instruction
                // KEY
                IRubyObject key = list.get(i++).interpret(runtime, context, self, aBlock);
                IRubyObject value = list.get(i++).interpret(runtime, context, self, aBlock);
   
                aset(runtime, hash, key, value);
            }
        }
      
        return hash;
    }
    
    protected void aset(Ruby runtime, RubyHash hash, IRubyObject key, IRubyObject value) {
        hash.fastASetCheckString(runtime, key, value);
    }
}
